"""Entrypoint of the CLI"""

import click

from ollama_manager.commands.delete import delete_model
from ollama_manager.commands.pull import pull_model
from ollama_manager.commands.run import run_model
from ollama_manager.commands.list import list_ollama_models
from ollama_manager.utils import coro


@click.group()
@coro
async def cli():
    pass


cli.add_command(pull_model)
cli.add_command(delete_model)
cli.add_command(run_model)
cli.add_command(list_ollama_models)



import os
import subprocess
import sys
from pathlib import Path

import click

from ollama_manager.utils import handle_interaction, list_models


def streamlit_check():
    try:
        import streamlit  # noqa F401
    except ImportError:
        print(
            """
⚠️ Running models with Streamlit UI is an optional feature.

To use it install the optional dependency:

>> pip install ollama-manager[ui]
"""
        )
        sys.exit(1)


@click.option(
    "--ui",
    "-ui",
    help="Run ollama models in a Streamlit UI, use either 'text' or 'vision'",
    type=str,
)
@click.command(name="run")
def run_model(ui: bool):
    """
    Run the selected Ollama model.
    By default, uses Ollama terminal UI.

    To run model in a streamlit UI:

    >> pip install ollama-manager[ui]

    ⚠️ Only text models are supported for now.
    """
    models = list_models()
    if ui.strip() not in ["text", "vision", ""]:
        print(
            f"❌ Invalid UI option: '{ui.strip()}'.\n"
            "Please use 'text' or 'vision'.\n\n"
            "⚠️ Only text models are supported for now."
        )
        sys.exit(1)

    if models:
        selection = handle_interaction(
            models, title="🚀 Select model to run:\n", multi_select=False
        )
    else:
        print("❌ No models selected for running with Streamlit UI")
        sys.exit(0)

    if selection:
        normalized_selection = selection[0].split()[0]
        if not ui:
            command = ["ollama", "run", normalized_selection]
        else:
            base_path = Path(os.path.abspath(__file__)).parent.parent / "ui"
            script_path = base_path / "text_chat.py"
            if ui.strip() == "text":
                script_path = base_path / "text_chat.py"
            elif ui.strip() == "vision":
                script_path = base_path / "image_chat.py"

            command = ["streamlit", "run", str(script_path), "--", normalized_selection]

        try:
            process = subprocess.Popen(command)
            process.wait()
        except Exception as e:
            print(f"Error running app: {e}")



import click
import ollama
from rich.console import Console
from rich.table import Table
from ollama_manager.utils import convert_bytes, humanized_relative_time


@click.command(name="list")
@click.option(
    "--sort-by",
    "-s",
    type=click.Choice(["name", "date", "size"]),
    default="name",
    help="Sort models by name, modified date, or size",
)
@click.option(
    "--direction",
    "-d",
    type=click.Choice(["asc", "desc"]),
    default="asc",
    help="Sort direction (ascending or descending)",
)
def list_ollama_models(sort_by, direction):
    """
    List your ollama models
    """
    model_metadata = ollama.list()
    models = model_metadata.models

    # Sort models based on user preference
    if sort_by == "name":
        models.sort(key=lambda m: m.model, reverse=(direction == "desc"))
    elif sort_by == "date":
        models.sort(key=lambda m: m.modified_at, reverse=(direction == "desc"))
    elif sort_by == "size":
        models.sort(key=lambda m: m.size, reverse=(direction == "desc"))

    console = Console()
    table = Table(title="Ollama Models")

    table.add_column("Model Name", style="bright_cyan")
    table.add_column("Modified Date", style="bright_green", justify="left")
    table.add_column("Size", style="bright_yellow", justify="left")

    # Add rows
    for model in models:
        model_name = model.model
        modified_date = humanized_relative_time(str(model.modified_at))
        size = convert_bytes(model.size)

        table.add_row(model_name, modified_date, size)

    console.print(table)



import os
import sys
import tempfile

import ollama
import streamlit as st
from PIL import Image

from ollama_manager.utils import list_models

st.set_page_config(
    page_title="Ollama Manager: Vision Chat", page_icon=":camera:", layout="wide"
)


def session_init():
    if "messages" not in st.session_state:
        st.session_state["messages"] = [
            {
                "role": "assistant",
                "content": "Hello, how can I help you?",
            }
        ]

    if "selected_model" not in st.session_state:
        st.session_state["selected_model"] = ""

    if "uploaded_image" not in st.session_state:
        st.session_state.uploaded_image = None


def sidebar():
    with st.sidebar:
        # Select Model
        models = list_models(only_names=True)
        selected_model = st.selectbox(
            ":material/precision_manufacturing: **:blue[Select Model]**",
            options=list_models(only_names=True),
            index=models.index(st.session_state["selected_model"] or sys.argv[1]),
        )

        if selected_model:
            st.session_state["selected_model"] = selected_model

        st.divider()

        # Upload Image
        st.title("Upload Image")
        uploaded_file = st.file_uploader(
            "Upload your image",
            type=["jpg", "jpeg", "png", "webp"],
            accept_multiple_files=False,
        )

        # Preview Uploaded Image
        if uploaded_file is not None:
            image = Image.open(uploaded_file)
            st.image(image, caption="Uploaded Image", use_container_width=True)

            with tempfile.NamedTemporaryFile(
                delete=False, suffix=os.path.splitext(uploaded_file.name)[1]
            ) as tmp_file:
                tmp_file.write(uploaded_file.getvalue())

                st.session_state.uploaded_image = tmp_file.name

        st.divider()
        st.info(
            """
        ### How to Use:
        1. Upload an image using the file uploader (Only single image upload is supported)
        2. Ask questions about the image in the chat
        3. Wait for the AI to respond
        """
        )


def call_llm():
    messages = st.session_state["messages"]
    stream = ollama.chat(
        model=st.session_state["selected_model"] or sys.argv[1],
        stream=True,
        messages=messages,
    )

    for chunk in stream:
        if chunk["done"] is False:
            yield chunk["message"]["content"]
        else:
            break


def run():
    session_init()
    sidebar()
    st.title("🤖 Ollama Manager: Vision Chat")

    for message in st.session_state["messages"]:
        message_role = message["role"]
        if message_role == "assistant":
            st.chat_message("assistant").write(message["content"])
        if message_role == "user":
            st.chat_message("human").write(message["content"])

    chat_input = st.chat_input(
        placeholder="Write your message...",
    )

    if chat_input:
        if st.session_state.uploaded_image is None:
            st.warning("Please upload an image first!")
        else:
            st.session_state.messages.append(
                {
                    "role": "user",
                    "content": chat_input,
                    "images": [st.session_state.uploaded_image],
                }
            )
            st.chat_message("human").write(chat_input)

            with st.spinner("Running..."):
                response = call_llm()
                with st.chat_message("assistant"):
                    ai_msg = st.write_stream(response)

                st.session_state["messages"] += [
                    {"role": "assistant", "content": ai_msg}
                ]


if __name__ == "__main__":
    run()



import sys

import click
import ollama

from ollama_manager.utils import handle_interaction, list_models


@click.command(name="rm")
@click.option(
    "--multi",
    "-m",
    help="Select multiple models at once",
    type=bool,
    default=False,
    is_flag=True,
)
@click.option(
    "--yes",
    "-y",
    help="Skip confirmation prompt for deletion",
    type=bool,
    default=False,
    is_flag=True,
)
def delete_model(multi: bool, yes: bool):
    """
    Deletes the selected model/s
    """
    models = list_models()
    if models:
        selections = handle_interaction(
            models, multi_select=multi, title="🗑️ Select model/s to delete:\n"
        )
    else:
        print("❌ No models selected for deletion")
        sys.exit(0)

    if selections:
        for selection in selections:
            normalized_selection = selection.split()[0]
            if not yes:
                confirm = input(
                    f"Are you sure you want to delete '\033[91m{normalized_selection}\033[0m'? \n[y(yes) | n(no)] "
                )

                if confirm.strip() in ("yes", "y"):
                    ollama.delete(normalized_selection)
                else:
                    print("❌ Exited delete mode.")
                    sys.exit(0)
            else:
                ollama.delete(normalized_selection)
            print(f"🗑️ Deleted model: {normalized_selection}")



import re
import sys

import click
import httpx
import ollama
from bs4 import BeautifulSoup, SoupStrainer
from rich.console import Console

from ollama_manager.utils import coro, handle_interaction, humanized_relative_time


def extract_quantization(text):
    """
    Extracts the quantization information from a model filename.

    Args:
        text: The model filename string.

    Returns:
        The quantization string (e.g., "Q2_K", "IQ4_K") or None if not found.
    """
    match = re.search(r"IQ\w+\.", text)  # Check for "IQ" first
    if match:
        return match.group(0)[:-1]
    match = re.search(r"Q\w+\.", text)  # Check for "Q" if "IQ" not found
    if match:
        return match.group(0)[:-1]
    return None


def format_bytes(size_bytes: int) -> str:
    """
    Formats a size in bytes to a human-readable string with suffix.

    Args:
        size_bytes: Integer representing size in bytes

    Returns:
        Formatted size string with appropriate suffix
    """
    # Precomputed suffix table with single-pass conversion
    _SUFFIXES = ("B", "KB", "MB", "GB", "TB", "PB")

    if not size_bytes:
        return "0B"

    # Bitwise optimization for log2-based scaling
    magnitude = (size_bytes.bit_length() - 1) // 10

    # Clamp magnitude to prevent index out of bounds
    magnitude = min(magnitude, len(_SUFFIXES) - 1)
    scaled_size = size_bytes / (1024**magnitude)
    return f"{scaled_size:.2f} {_SUFFIXES[magnitude]}"


async def list_remote_model_tags(model_name: str, client: httpx.AsyncClient):
    response = await client.get(f"https://ollama.com/library/{model_name}/tags")
    soup = BeautifulSoup(response.text, "html.parser")

    model_entries = soup.select("div.group")
    if not model_entries:
        return []

    tags = []
    for entry in model_entries:
        # Extract model name and tag
        name_element = entry.select_one("span.group-hover\\:underline")
        if not name_element:
            continue

        full_name = name_element.text.strip()
        tag_parts = full_name.split(":")
        tag_name = tag_parts[1] if len(tag_parts) > 1 else "latest"

        size_element = entry.select_one("p.col-span-2") or entry.select_one(
            "div.text-neutral-500"
        )
        size = None
        if size_element:
            size_text = size_element.get_text(strip=True)
            size_match = re.search(r"(\d+(?:\.\d+)?[GMKT]B)", size_text)
            if size_match:
                size = size_match.group(1)

        context_element = (
            entry.select("p.col-span-2")[1]
            if len(entry.select("p.col-span-2")) > 1
            else None
        )
        context_window = None
        if context_element:
            context_window = context_element.get_text(strip=True)
        else:
            all_text = entry.get_text(separator=" ", strip=True)
            context_match = re.search(r"(\d+[KM]?) context window", all_text)
            if context_match:
                context_window = context_match.group(1)

        input_element = entry.select_one("div.col-span-2") or entry.select_one(
            "div.text-neutral-500"
        )
        input_types = []
        if input_element:
            input_text = input_element.get_text(strip=True)
            if "Text" in input_text:
                input_types.append("Text")
            if "Vision" in input_text:
                input_types.append("Vision")

        timestamp_element = entry.select_one(
            "div.flex.text-neutral-500.text-xs"
        ) or entry.select_one("div.flex.sm\\:hidden")
        updated = None
        if timestamp_element:
            timestamp_text = timestamp_element.get_text(strip=True)
            time_match = re.search(r"(\d+\s+\w+\s+ago)", timestamp_text)
            if time_match:
                updated = time_match.group(1)

        hash_element = entry.select_one("span.font-mono")
        hash_id = hash_element.get_text(strip=True) if hash_element else None

        tags.append(
            {
                "title": tag_name,
                "size": size,
                "context_window": context_window,
                "input_types": input_types,
                "updated": updated,
                "hash": hash_id,
            }
        )

    return tags


async def list_remote_models(client: httpx.AsyncClient) -> list[str] | None:
    response = await client.get(url="https://ollama.com/search")

    title_strainer = SoupStrainer("span", attrs={"x-test-search-response-title": True})
    soup = BeautifulSoup(response.text, "html.parser", parse_only=title_strainer)
    elements = soup.find_all("span", attrs={"x-test-search-response-title": True})

    if not elements:
        return None

    return [element.text.strip() for element in elements]


async def list_hugging_face_models(
    client: httpx.AsyncClient, limit: int, query: str
) -> list[dict[str, str]]:
    BASE_API_ENDPOINT = "https://huggingface.co/api/models"
    params = {
        "filter": "gguf",
        "sort": "downloads",
        "direction": "-1",
        "limit": limit,
        "full": False,
        "config": False,
        "search": query,
    }
    try:
        res = await client.get(url=BASE_API_ENDPOINT, params=params)
    except Exception:
        print(
            "❌ Failed fetching model from Hugging Face.\n>>> 🔁 Try again\n>>> 🛜 Make sure you are connected to the internet."
        )
        sys.exit(1)
    hf_response = res.json()
    payload = []

    if not hf_response:
        print(f"❌ Model not found: {query}")
        sys.exit(1)

    for response in hf_response:
        payload.append(response.get("modelId"))
    return payload


async def list_hugging_face_model_quantization(
    client: httpx.AsyncClient, model_name: str
):
    try:
        res = await client.get(
            url=f"https://huggingface.co/api/models/{model_name}?blobs=true"
        )
    except Exception:
        print(
            "❌ Failed fetching model from Hugging Face.\n>>> 🔁 Try again\n>>> 🛜 Make sure you are connected to the internet."
        )
        sys.exit(1)
    hf_response = res.json()
    payload = []
    files = hf_response.get("siblings")
    last_modified = humanized_relative_time(hf_response.get("lastModified"))
    for file in files:
        filename = file.get("rfilename")
        if filename.endswith(".gguf"):
            quantization = extract_quantization(filename)
            if not quantization:
                continue

            payload.append(
                {
                    "title": quantization,
                    "size": format_bytes(file.get("size")),
                    "updated": last_modified,
                }
            )

    return payload


@click.command(name="pull")
@click.option(
    "--hugging_face",
    "-hf",
    help="Pull models from Hugging Face",
    type=bool,
    default=False,
    is_flag=True,
)
@click.option("--query", "-q", help="Query for hugging face model", type=str)
@click.option(
    "--limit",
    "-l",
    help="Limit the number of output from hugging face. Default is 20",
    type=int,
    default=20,
)
@coro
async def pull_model(hugging_face: bool, query: str, limit: int):
    """
    Pull models from Ollama library:

    https://ollama.dev/search
    """

    console = Console()
    async with httpx.AsyncClient() as client:
        if hugging_face:
            if not query:
                query = input("🤗 hf search: ")

            with console.status("Fetching models from Hugging Face", spinner="dots"):
                models = await list_hugging_face_models(client, limit, query)
        else:
            with console.status(
                "Fetching models from Ollama directory", spinner="dots"
            ):
                models = await list_remote_models(client)

        if not models:
            print("❌ No models selected for download")
            sys.exit(0)

        model_selection = handle_interaction(
            models, title="📦 Select remote Ollama model\s:\n", multi_select=False
        )
        if model_selection:
            if hugging_face:
                with console.status("Fetching quantization levels", spinner="dots"):
                    model_tags = await list_hugging_face_model_quantization(
                        client=client, model_name=model_selection[0]
                    )
            else:
                # with console.status("Fetching model tags", spinner="dots"):
                model_tags = await list_remote_model_tags(
                    model_name=model_selection[0], client=client
                )
            if not model_tags:
                print(
                    f"❌ Failed fetching tags for: {model_selection}. Please try again."
                )
                sys.exit(1)

            title_max = size_max = context_window_max = input_type_max = updated_max = 0
            COLUMN_PADDING = 8
            for tag in model_tags:
                title_max = max(title_max, len(tag.get("title", "")))
                size_max = max(size_max, len(tag.get("size", "")))
                context_window_max = max(
                    context_window_max, len(tag.get("context_window", ""))
                )
                input_type_max = max(input_type_max, len(tag.get("input_types", "")))
                updated_max = max(updated_max, len(tag.get("updated", "")))

            if hugging_face:
                model_name_with_tags = [
                    f"{tag['title']:<{title_max + COLUMN_PADDING}}{tag['size']:<{size_max +COLUMN_PADDING}}{tag['updated']}"
                    for tag in model_tags
                ]
            else:
                model_name_with_tags = []
                for tag in model_tags:
                    display = f"{model_selection[0]}:{tag['title']:<{title_max + COLUMN_PADDING}}{tag['size']:<{size_max+COLUMN_PADDING}}"

                    if tag.get("context_window"):
                        display += f"{tag['context_window']:<{context_window_max + COLUMN_PADDING}}"

                    if tag.get("input_types"):
                        display += f"{','.join(tag['input_types']):<{input_type_max + COLUMN_PADDING}}"

                    if tag.get("updated"):
                        display += f"{tag['updated']}"

                    model_name_with_tags.append(display)
            selected_model_with_tag = handle_interaction(
                model_name_with_tags, title="🔖 Select tag/quantization:\n"
            )
            if not selected_model_with_tag:
                print("No tag selected for the model")
                sys.exit(1)

            if hugging_face:
                final_model = (
                    f"hf.co/{model_selection[0]}:{model_name_with_tags[0]}".split()[0]
                )
            else:
                final_model = selected_model_with_tag[0].split()[0]
            print(f">>> Pulling model: {final_model}")
            try:
                response = ollama.pull(final_model, stream=True)
                screen_padding = 100

                for data in response:
                    out = f"Status: {data.get('status')} | Completed: {format_bytes(data.get('completed'))}/{format_bytes(data.get('total'))}"
                    print(f"{out:<{screen_padding}}", end="\r", flush=True)

                print(f"\r{' ' * screen_padding}\r")  # Clear screen
                print(f"✅ {final_model} model is ready for use!\n\n>>> olm run\n")
            except Exception as e:
                print(f"❌ Failed downloading {final_model}\n{str(e)}")
